"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const chrome_remote_interface_1 = __importDefault(require("chrome-remote-interface"));
const commander_1 = require("commander");
const entrypoint = __dirname + '/../http-dist/index.html';
async function main() {
    commander_1.program
        .addOption(new commander_1.Option('--url <url>', 'URL of the CrossLab instance').env('CROSSLAB_URL'))
        .addOption(new commander_1.Option('--auth-token <token>', 'Token to use for authentication').env('CROSSLAB_TOKEN'))
        .addOption(new commander_1.Option('--device-url <device-url>', 'Device url').env('CROSSLAB_DEVICE_URL'))
        .addOption(new commander_1.Option('--browser-inspect <port>', 'Chrome Debug Port'));
    commander_1.program.parse();
    const opts = commander_1.program.opts();
    const options = {
        baseUrl: opts.url,
        authToken: opts.authToken,
        deviceUrl: opts.deviceUrl,
    };
    const chromePort = opts.browserInspect ?? Math.floor(Math.random() * 10000) + 10000;
    const debugging = opts.browserInspect !== undefined;
    const chromium = (0, child_process_1.spawn)('chromium', [
        '--headless',
        '--no-sandbox',
        '--disable-gpu',
        '--disable-dev-shm-usage',
        '--enable-logging=stderr',
        '--vmodule=*/webrtc/*=2',
        '--remote-debugging-port=' + chromePort,
        entrypoint,
    ], { env: {} });
    chromium.stderr.on('data', (data) => {
        console.log(data.toString());
    });
    chromium.stdout.on('data', (data) => {
        console.log(data.toString());
    });
    await new Promise(resolve => {
        chromium.stderr.on('data', (data) => {
            if (data.includes('DevTools listening on ws://')) {
                resolve();
            }
        });
    });
    console.log('Chromium is ready');
    await new Promise(resolve => setTimeout(resolve, 1000)); // wait for target to be ready
    const protocol = await (0, chrome_remote_interface_1.default)({ port: chromePort });
    const { Debugger, Runtime, Log } = protocol;
    await Runtime.enable();
    await Log.enable();
    await Debugger.enable();
    Log.entryAdded(result => {
        console.log(result.entry.text);
    });
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    const previewTransform = (preview) => {
        if ('value' in preview) {
            return preview.value;
        }
        else if (preview.type === 'object') {
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            const ret = {};
            for (const prop of preview.properties) {
                ret[prop.name] = previewTransform(prop);
            }
            return ret;
        }
        else {
            return undefined;
        }
    };
    Runtime.consoleAPICalled(result => {
        console.log(...result.args.map(arg => arg.value ?? previewTransform(arg.preview)));
    });
    Runtime.exceptionThrown(async (event) => {
        console.log(event.exceptionDetails);
        await protocol.close();
        chromium.kill('SIGKILL');
        process.exit();
    });
    await new Promise(resolve => setTimeout(resolve, 500));
    debugging && (await Debugger.pause());
    const expression = 'window.app(' + JSON.stringify(options) + ')';
    const appPromise = await (await Runtime.evaluate({ expression, silent: false })).result;
    if (appPromise.objectId === undefined) {
        throw new Error('app() did not return a promise');
    }
    //debugging && console.log(await Debugger.paused());
    //debugging && await Debugger.stepInto();
    console.log(await Runtime.awaitPromise({ promiseObjectId: appPromise.objectId }));
    let stdin = '';
    process.stdin.on('data', data => {
        stdin += data.toString();
        const lines = stdin.split('\n');
        stdin = lines.pop() ?? '';
        for (const line of lines) {
            // eslint-disable-next-line prefer-const
            let [event, argument] = line.split(' ');
            event = event.slice(1, -1);
            const eventExpression = 'window.event("' + event + '",' + argument + ')';
            Runtime.evaluate({ expression: eventExpression, silent: false });
        }
    });
    console.log('[ready]');
    await new Promise(resolve => {
        process.on('SIGINT', () => {
            resolve();
        });
    });
    console.log('[closed]');
    await protocol.close();
    chromium.kill('SIGKILL');
    process.exit();
}
main();
//# sourceMappingURL=index.js.map